@@ -32,7 +32,7 @@
  */
 
 #include "ftp_locl.h"
-RCSID ("$Id: ftp.c,v 1.57 1999/08/03 12:02:18 assar Exp $");
+RCSID ("$Id: ftp.c,v 1.63 2000/01/08 07:43:47 assar Exp $");
 
 struct sockaddr_storage hisctladdr_ss;
 struct sockaddr *hisctladdr = (struct sockaddr *)&hisctladdr_ss;
@@ -55,62 +55,59 @@ typedef void (*sighand) (int);
 char *
 hookup (const char *host, int port)
 {
-    struct hostent *hp = NULL;
-    int s, len;
     static char hostnamebuf[MaxHostNameLen];
+    struct addrinfo *ai, *a;
+    struct addrinfo hints;
     int error;
-    int af;
-    char **h;
-    int ret;
+    char portstr[NI_MAXSERV];
+    int len;
+    int s;
 
-#ifdef HAVE_IPV6
-    if (hp == NULL)
-	hp = getipnodebyname (host, AF_INET6, 0, &error);
-#endif
-    if (hp == NULL)
-	hp = getipnodebyname (host, AF_INET, 0, &error);
+    memset (&hints, 0, sizeof(hints));
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_protocol = IPPROTO_TCP;
+    hints.ai_flags    = AI_CANONNAME;
 
-    if (hp == NULL) {
-	warnx ("%s: %s", host, hstrerror(error));
+    snprintf (portstr, sizeof(portstr), "%u", ntohs(port));
+
+    error = getaddrinfo (host, portstr, &hints, &ai);
+    if (error) {
+	warnx ("%s: %s", host, gai_strerror(error));
 	code = -1;
 	return NULL;
     }
-    strcpy_truncate (hostnamebuf, hp->h_name, sizeof(hostnamebuf));
+    strlcpy (hostnamebuf, host, sizeof(hostnamebuf));
     hostname = hostnamebuf;
-    af = hisctladdr->sa_family = hp->h_addrtype;
 
-    for (h = hp->h_addr_list;
-	 *h != NULL;
-	 ++h) {
+    for (a = ai; a != NULL; a = a->ai_next) {
+	s = socket (a->ai_family, a->ai_socktype, a->ai_protocol);
+	if (s < 0)
+	    continue;
+
+	if (a->ai_canonname != NULL)
+	    strlcpy (hostnamebuf, a->ai_canonname, sizeof(hostnamebuf));
+
+	memcpy (hisctladdr, a->ai_addr, a->ai_addrlen);
 	
-	s = socket (af, SOCK_STREAM, 0);
-	if (s < 0) {
-	    warn ("socket");
-	    code = -1;
-	    freehostent (hp);
-	    return (0);
-	}
+	error = connect (s, a->ai_addr, a->ai_addrlen);
+	if (error < 0) {
+	    char addrstr[256];
 
-	socket_set_address_and_port (hisctladdr, *h, port);
-
-	ret = connect (s, hisctladdr, socket_sockaddr_size(hisctladdr));
-	if (ret < 0) {
-	    char addr[256];
-
-	    if (inet_ntop (af, socket_get_address(hisctladdr),
-			   addr, sizeof(addr)) == NULL)
-		strcpy_truncate (addr, "unknown address",
-				 sizeof(addr));
-	    warn ("connect %s", addr);
+	    if (getnameinfo (a->ai_addr, a->ai_addrlen,
+			     addrstr, sizeof(addrstr),
+			     NULL, 0, NI_NUMERICHOST) != 0)
+		strlcpy (addrstr, "unknown address", sizeof(addrstr));
+			     
+	    warn ("connect %s", addrstr);
 	    close (s);
 	    continue;
 	}
 	break;
     }
-    freehostent (hp);
-    if (ret < 0) {
+    freeaddrinfo (ai);
+    if (error < 0) {
+	warnx ("failed to contact %s", host);
 	code = -1;
-	close (s);
 	return NULL;
     }
 
@@ -200,7 +197,7 @@ login (char *host)
 	else
 	    user = tmp;
     }
-    strcpy_truncate(username, user, sizeof(username));
+    strlcpy(username, user, sizeof(username));
     n = command("USER %s", user);
     if (n == CONTINUE) {
 	if(sec_complete)
@@ -240,7 +237,7 @@ login (char *host)
 	return (1);
     for (n = 0; n < macnum; ++n) {
 	if (!strcmp("init", macros[n].mac_name)) {
-	    strcpy_truncate (line, "$init", sizeof (line));
+	    strlcpy (line, "$init", sizeof (line));
 	    makeargv();
 	    domacro(margc, margv);
 	    break;
@@ -567,6 +564,11 @@ copy_stream (FILE * from, FILE * to)
 #endif
 
     if (fstat (fileno (from), &st) == 0 && S_ISREG (st.st_mode)) {
+	/*
+	 * mmap zero bytes has potential of loosing, don't do it.
+	 */
+	if (st.st_size == 0)
+	    return 0;
 	chunk = mmap (0, st.st_size, PROT_READ, MAP_SHARED, fileno (from), 0);
 	if (chunk != (void *) MAP_FAILED) {
 	    int res;
@@ -1355,7 +1357,7 @@ initconn (void)
 }
 
 FILE *
-dataconn (char *lmode)
+dataconn (const char *lmode)
 {
     struct sockaddr_storage from_ss;
     struct sockaddr *from = (struct sockaddr *)&from_ss;
@@ -1466,7 +1468,7 @@ pswitch (int flag)
     ip->connect = connected;
     connected = op->connect;
     if (hostname) {
-	strcpy_truncate (ip->name, hostname, sizeof (ip->name));
+	strlcpy (ip->name, hostname, sizeof (ip->name));
     } else
 	ip->name[0] = 0;
     hostname = op->name;
@@ -1492,16 +1494,16 @@ pswitch (int flag)
     mcase = op->mcse;
     ip->ntflg = ntflag;
     ntflag = op->ntflg;
-    strcpy_truncate (ip->nti, ntin, sizeof (ip->nti));
-    strcpy_truncate (ntin, op->nti, 17);
-    strcpy_truncate (ip->nto, ntout, sizeof (ip->nto));
-    strcpy_truncate (ntout, op->nto, 17);
+    strlcpy (ip->nti, ntin, sizeof (ip->nti));
+    strlcpy (ntin, op->nti, 17);
+    strlcpy (ip->nto, ntout, sizeof (ip->nto));
+    strlcpy (ntout, op->nto, 17);
     ip->mapflg = mapflag;
     mapflag = op->mapflg;
-    strcpy_truncate (ip->mi, mapin, MaxPathLen);
-    strcpy_truncate (mapin, op->mi, MaxPathLen);
-    strcpy_truncate (ip->mo, mapout, MaxPathLen);
-    strcpy_truncate (mapout, op->mo, MaxPathLen);
+    strlcpy (ip->mi, mapin, MaxPathLen);
+    strlcpy (mapin, op->mi, MaxPathLen);
+    strlcpy (ip->mo, mapout, MaxPathLen);
+    strlcpy (mapout, op->mo, MaxPathLen);
     signal(SIGINT, oldintr);
     if (abrtflag) {
 	abrtflag = 0;
@@ -1675,7 +1677,7 @@ gunique (char *local)
 	warn ("local: %s", local);
 	return NULL;
     }
-    strcpy_truncate (new, local, sizeof(new));
+    strlcpy (new, local, sizeof(new));
     cp = new + strlen(new);
     *cp++ = '.';
     while (!d) {
