@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #include "kx.h"
 
-RCSID("$Id: common.c,v 1.50 1999/05/12 17:33:39 assar Exp $");
+RCSID("$Id: common.c,v 1.58 2000/01/08 08:02:46 assar Exp $");
 
 char x_socket[MaxPathLen];
 
@@ -58,10 +53,6 @@ size_t cookie_len = sizeof(cookie);
 #define X_PIPE_PATH "/tmp/.X11-pipe/X"
 #endif
 
-#ifndef INADDR_LOOPBACK
-#define INADDR_LOOPBACK 0x7f000001
-#endif
-
 /*
  * Allocate a unix domain socket in `s' for display `dpy' and with
  * filename `pattern'
@@ -416,7 +407,7 @@ create_and_write_cookie (char *xauthfile,
      auth.data = (char*)cookie;
      des_rand_data (cookie, cookie_sz);
 
-     strcpy_truncate(xauthfile, "/tmp/AXXXXXX", xauthfile_size);
+     strlcpy(xauthfile, "/tmp/AXXXXXX", xauthfile_size);
      fd = mkstemp(xauthfile);
      if(fd < 0) {
 	 saved_errno = errno;
@@ -569,10 +560,11 @@ fail:
  */
 
 static int
-match_local_auth (Xauth* auth, struct hostent *disp_he, int disp_nr)
+match_local_auth (Xauth* auth, struct addrinfo *ai, int disp_nr)
 {
     int auth_disp;
     char *tmp_disp;
+    struct addrinfo *a;
     
     tmp_disp = strndup (auth->number, auth->number_length);
     if (tmp_disp == NULL)
@@ -581,29 +573,14 @@ match_local_auth (Xauth* auth, struct hostent *disp_he, int disp_nr)
     free (tmp_disp);
     if (auth_disp != disp_nr)
 	return 1;
-    if (auth->family == FamilyLocal) {
-	int i;
-
-	if (strncmp (auth->address,
-		     disp_he->h_name,
-		     auth->address_length) == 0)
-	    return 0;
-
-	for (i = 0; disp_he->h_aliases[i] != NULL; ++i)
-	    if (strncmp (auth->address,
-			 disp_he->h_aliases[i],
-			 auth->address_length) == 0)
-		return 0;
-    } else if (auth->family == FamilyInternet
-	     && disp_he->h_addrtype == AF_INET
-	     && auth->address_length == disp_he->h_length) {
-	int i;
-
-	for (i = 0; disp_he->h_addr_list[i] != NULL; ++i)
-	    if (memcmp (disp_he->h_addr_list[i],
-			auth->address,
+    for (a = ai; a != NULL; a = a->ai_next) {
+	if ((auth->family == FamilyLocal
+	     || auth->family == FamilyWild)
+	    && a->ai_canonname != NULL
+	    && strncmp (auth->address,
+			a->ai_canonname,
 			auth->address_length) == 0)
-		return 0;
+	    return 0;
     }
     return 1;
 }
@@ -619,8 +596,10 @@ find_auth_cookie (FILE *f)
     char local_hostname[MaxHostNameLen];
     char *display = getenv("DISPLAY");
     char *colon;
-    struct hostent *display_he;
+    struct addrinfo *ai;
+    struct addrinfo hints;
     int disp;
+    int error;
 
     if (display == NULL)
 	display = ":0";
@@ -637,16 +616,24 @@ find_auth_cookie (FILE *f)
 	gethostname (local_hostname, sizeof(local_hostname));
 	display = local_hostname;
     }
-    display_he = gethostbyname (display);
-    if (display_he == NULL) {
-	warnx ("gethostbyname %s: %s", display, hstrerror(h_errno));
+    memset (&hints, 0, sizeof(hints));
+    hints.ai_flags    = AI_CANONNAME;
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_protocol = IPPROTO_TCP;
+
+    error = getaddrinfo (display, NULL, &hints, &ai);
+    if (error) {
+	warnx ("getaddrinfo %s: %s", display, gai_strerror(error));
 	return NULL;
     }
 
     for (; (ret = XauReadAuth (f)) != NULL; XauDisposeAuth(ret)) {
-	if (match_local_auth (ret, display_he, disp) == 0)
+	if (match_local_auth (ret, ai, disp) == 0) {
+	    freeaddrinfo (ai);
 	    return ret;
+	}
     }
+    freeaddrinfo (ai);
     return NULL;
 }
 
